@@ -84,18 +84,17 @@ class Token(object):
         # Token-related attributes
         self.id = auth_ref.auth_token
         self.unscoped_token = unscoped_token
-        if (_TOKEN_HASH_ENABLED and
-                (keystone_cms.is_asn1_token(self.id)
-                    or keystone_cms.is_pkiz(self.id))):
+        if _TOKEN_HASH_ENABLED and self._is_pki_token(self.id):
             algorithm = getattr(settings, 'OPENSTACK_TOKEN_HASH_ALGORITHM',
                                 'md5')
             hasher = hashlib.new(algorithm)
             hasher.update(self.id)
             self.id = hasher.hexdigest()
-            # If the scoped_token is long, then unscoped_token must be too.
-            hasher = hashlib.new(algorithm)
-            hasher.update(self.unscoped_token)
-            self.unscoped_token = hasher.hexdigest()
+            # Only hash unscoped token if needed
+            if self._is_pki_token(self.unscoped_token):
+                hasher = hashlib.new(algorithm)
+                hasher.update(self.unscoped_token)
+                self.unscoped_token = hasher.hexdigest()
         self.expires = auth_ref.expires
 
         # Project-related attributes
@@ -116,6 +115,11 @@ class Token(object):
         self.roles = [{'name': role} for role in auth_ref.role_names]
         self.serviceCatalog = auth_ref.service_catalog.catalog
 
+    def _is_pki_token(self, token):
+        """Determines if this is a pki-based token (pki or pkiz)"""
+        return (keystone_cms.is_ans1_token(token)
+                or keystone_cms.is_pkiz(token))
+
 
 class User(models.AbstractBaseUser, models.AnonymousUser):
     """A User class with some extra special sauce for Keystone.
@@ -293,10 +297,7 @@ class User(models.AbstractBaseUser, models.AnonymousUser):
 
         Returns ``True`` or ``False``.
         """
-        admin_roles = {role.lower() for role in getattr(
-            settings,
-            'OPENSTACK_KEYSTONE_ADMIN_ROLES',
-            ['admin'])}
+        admin_roles = utils.get_admin_roles()
         user_roles = {role['name'].lower() for role in self.roles}
         return not admin_roles.isdisjoint(user_roles)
 
@@ -400,3 +401,6 @@ class User(models.AbstractBaseUser, models.AnonymousUser):
                 if not self.has_a_matching_perm(perm, obj):
                     return False
         return True
+
+    class Meta(object):
+        app_label = 'openstack_auth'
